package App::lcpan::CmdBundle::ver;

our $DATE = '2017-01-20'; # DATE
our $VERSION = '0.03'; # VERSION

1;
# ABSTRACT: lcpan subcommands related to module versions

__END__

=pod

=encoding UTF-8

=head1 NAME

App::lcpan::CmdBundle::ver - lcpan subcommands related to module versions

=head1 VERSION

This document describes version 0.03 of App::lcpan::CmdBundle::ver (from Perl distribution App-lcpan-CmdBundle-ver), released on 2017-01-20.

=head1 SYNOPSIS

=head1 DESCRIPTION

This bundle provides the following lcpan subcommands:

=over

=item * L<lcpan ver-cmp-list|App::lcpan::Cmd::ver_cmp_list>

=item * L<lcpan outdated|App::lcpan::Cmd::outdated>

=item * L<lcpan ver-cmp-installed|App::lcpan::Cmd::ver_cmp_installed>

=back

This distribution is experimental. Subcommands might be renamed and later merged
to the main App-lcpan distribution.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/App-lcpan-CmdBundle-ver>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-App-lcpan-CmdBundle-ver>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=App-lcpan-CmdBundle-ver>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 SEE ALSO

L<lcpan>

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
